from smart_module.module_system.header_dialogs import *
from smart_module.module_system.module_scripts import *

'''
    超级功能
    1.招募任何队伍
    2.和同伴对话移动到任何据点附近
'''

superFunction = {
    "name":"SuperFunction",
    "enable":True,
    "version": "v2.0.0",
    "desc":'''
        超级功能
        1.招募任何队伍
        2.和同伴对话移动到任何据点附近
    ''',
    "actions": [
        ("Prepend@dialogs|party_encounter_hostile_defender&1:Nothing%^&3:close_window&4",[
            ## 任何人员加入玩家
            [anyone|plyr, "party_encounter_hostile_defender", [], "I order you to join me.", "super_function_recruit_start", []],
            [anyone, "super_function_recruit_start", [], "yes,my lord!", "close_window", [
                (assign, "$g_leave_encounter",1),
                (call_script,"script_add_party_as_companions","p_main_party","$g_encountered_party",1),
            ]],
        ]),
        ("Prepend@dialogs|lord_talk&1:I_must_beg_my_leave%^&3:lord_leave&4",[
            ## 任何人员加入玩家
            [anyone|plyr, "lord_talk", [], "I order you to join me.", "super_function_recruit_start", []],
            [anyone, "super_function_recruit_start", [], "yes,my lord!", "close_window", [
                (assign, "$g_leave_encounter", 1),
                (call_script,"script_add_party_as_companions","p_main_party","$g_encountered_party",1),
            ]],
        ]),
        ("Prepend@dialogs|member_talk&1:Never_mind_%^&3:close_window&4",[
            ## 任何人员加入玩家
            [anyone|plyr, "member_talk", [], "I want to teleport somewhere.", "super_function_teleport_start", []],
            [anyone, "super_function_teleport_start", [], "What kind of place?", "super_function_teleport_center_type", []],
            [anyone|plyr|repeat_for_100, "super_function_teleport_center_type", [
                (store_repeat_object,":typeIndex"),
                (is_between,":typeIndex",0,4),
                (str_clear,s0),
                (try_begin),
                    (eq,":typeIndex",0),
                    (str_store_string,s0,"@town"),
                (else_try),
                    (eq,":typeIndex",1),
                    (str_store_string,s0,"@castle"),
                (else_try),
                    (eq,":typeIndex",2),
                    (str_store_string,s0,"@village"),
                (else_try),
                    (eq,":typeIndex",3),
                    (str_store_string,s0,"@training_ground_place"),
                (try_end),
            ], "{s0}", "super_function_teleport_center_choice_start", [
                (store_repeat_object,":typeIndex"),
                (str_clear,s0),
                (try_begin),
                    (eq,":typeIndex",0),
                    (assign,"$g_teleport_center_begin",towns_begin),
                    (assign,"$g_teleport_center_end",towns_end),
                    (str_store_string,s0,"@town"),
                (else_try),
                    (eq,":typeIndex",1),
                    (assign, "$g_teleport_center_begin", castles_begin),
                    (assign, "$g_teleport_center_end", castles_end),
                    (str_store_string,s0,"@castle"),
                (else_try),
                    (eq,":typeIndex",2),
                    (assign, "$g_teleport_center_begin", villages_begin),
                    (assign, "$g_teleport_center_end", villages_end),
                    (str_store_string,s0,"@village"),
                (else_try),
                    (eq,":typeIndex",3),
                    (assign, "$g_teleport_center_begin", training_grounds_begin),
                    (assign, "$g_teleport_center_end", training_grounds_end),
                    (str_store_string,s0,"@training_ground_place"),
                (try_end),
            ]],

            [anyone, "super_function_teleport_center_choice_start", [], "{s0}", "super_function_teleport_center_choice_center", []],
            ## 地点选择
            [anyone|plyr|repeat_for_parties, "super_function_teleport_center_choice_center", [
                (store_repeat_object,":centerNo"),
                (is_between,":centerNo","$g_teleport_center_begin","$g_teleport_center_end"),
                (str_store_party_name,s1,":centerNo"),
                (store_faction_of_party,":faction",":centerNo"),
                (str_store_faction_name,s2,":faction"),
            ], "{s0}:{s1}({s2})", "close_window", [
                (store_repeat_object,":centerNo"),
                (party_relocate_near_party,"p_main_party",":centerNo",3),
                (str_store_party_name,s1,":centerNo"),
                (display_message,"@teleport to {s1}"),
            ]],

            [anyone, "super_function_teleport_center_choice_center", [], "no", "close_window", []],

            [anyone, "super_function_teleport_start", [], "nothing.", "close_window", []],
        ]),
    ],
    "internationals":{
        "cns":{
            "dialogs":[
                "dlga_party_encounter_hostile_defender:super_function_recruit_start|【 超 级 功 能 】我 命 令 你 加 入 我 !",
                "dlga_super_function_recruit_start:close_window|好 的 ，大 人 !",
                "dlga_lord_talk:super_function_recruit_start|我 命 令 你 加 入 我 !",
                ## "dlga_super_function_recruit_start:close_window|好 的 ，大 人 !",


                "dlga_member_talk:super_function_teleport_start|【 超 级 功 能 】我 想 传 送 到 某 个 地 方 !",
                "dlga_super_function_teleport_start:super_function_teleport_center_type|什 么 类 型 的 地 方 ？",
            ],
            "quick_strings":[
                "qstr_training_ground_place|训 练 场",
                "qstr_teleport_to_{s1}|传 送 到 : {s1}"
            ]
        }
    },
}


import string
import types

def convert_to_identifier(s0):
  s1 = s0.replace(" ","_")
  s2 = s1.replace("'","_")
  s3 = s2.replace("`","_")
  s4 = s3.replace("(","_")
  s5 = s4.replace(")","_")
  s6 = s5.replace("-","_")
  s7 = s6.replace(",","")
  s8 = s7.replace("|","")
  s9 = s8.replace("\t","_") #Tab
  s10 = s9.lower()
  return s10

def convert_to_identifier_with_no_lowercase(s0):
  s1 = s0.replace(" ","_")
  s2 = s1.replace("'","_")
  s3 = s2.replace("`","_")
  s4 = s3.replace("(","_")
  s5 = s4.replace(")","_")
  s6 = s5.replace("-","_")
  s7 = s6.replace(",","")
  s8 = s7.replace("|","")
  s9 = s8.replace("\t","_") #Tab
  return s9

def replace_spaces(s0):
  return s0.replace("\t","_").replace(" ","_")



psys_game_rain = 0
psys_game_snow = 1
psys_game_blood = 2
psys_game_blood_2 = 3
psys_game_hoof_dust = 4
psys_game_hoof_dust_snow = 5
psys_game_hoof_dust_mud = 6
psys_game_water_splash_1 = 7
psys_game_water_splash_2 = 8
psys_game_water_splash_3 = 9
psys_torch_fire = 10
psys_fire_glow_1 = 11
psys_fire_glow_fixed = 12
psys_torch_smoke = 13
psys_flue_smoke_short = 14
psys_flue_smoke_tall = 15
psys_war_smoke_tall = 16
psys_ladder_dust_6m = 17
psys_ladder_dust_8m = 18
psys_ladder_dust_10m = 19
psys_ladder_dust_12m = 20
psys_ladder_dust_14m = 21
psys_ladder_straw_6m = 22
psys_ladder_straw_8m = 23
psys_ladder_straw_10m = 24
psys_ladder_straw_12m = 25
psys_ladder_straw_14m = 26
psys_torch_fire_sparks = 27
psys_fire_sparks_1 = 28
psys_pistol_smoke = 29
psys_brazier_fire_1 = 30
psys_cooking_fire_1 = 31
psys_cooking_smoke = 32
psys_food_steam = 33
psys_candle_light = 34
psys_candle_light_small = 35
psys_lamp_fire = 36
psys_dummy_smoke = 37
psys_dummy_straw = 38
psys_dummy_smoke_big = 39
psys_dummy_straw_big = 40
psys_gourd_smoke = 41
psys_gourd_piece_1 = 42
psys_gourd_piece_2 = 43
psys_fire_fly_1 = 44
psys_bug_fly_1 = 45
psys_moon_beam_1 = 46
psys_moon_beam_paricle_1 = 47
psys_night_smoke_1 = 48
psys_fireplace_fire_small = 49
psys_fireplace_fire_big = 50
psys_village_fire_big = 51
psys_village_fire_smoke_big = 52
psys_map_village_fire = 53
psys_map_village_fire_smoke = 54
psys_map_village_looted_smoke = 55
psys_dungeon_water_drops = 56
psys_wedding_rose = 57
psys_sea_foam_a = 58
psys_fall_leafs_a = 59



tableau_game_character_sheet = 0
tableau_game_inventory_window = 1
tableau_game_profile_window = 2
tableau_game_party_window = 3
tableau_game_troop_label_banner = 4
tableau_round_shield_1 = 5
tableau_round_shield_2 = 6
tableau_round_shield_3 = 7
tableau_round_shield_4 = 8
tableau_round_shield_5 = 9
tableau_small_round_shield_1 = 10
tableau_small_round_shield_2 = 11
tableau_small_round_shield_3 = 12
tableau_kite_shield_1 = 13
tableau_kite_shield_2 = 14
tableau_kite_shield_3 = 15
tableau_kite_shield_4 = 16
tableau_heater_shield_1 = 17
tableau_heater_shield_2 = 18
tableau_pavise_shield_1 = 19
tableau_pavise_shield_2 = 20
tableau_heraldic_armor_a = 21
tableau_heraldic_armor_b = 22
tableau_heraldic_armor_c = 23
tableau_heraldic_armor_d = 24
tableau_troop_note_alpha_mask = 25
tableau_troop_note_color = 26
tableau_troop_character_alpha_mask = 27
tableau_troop_character_color = 28
tableau_troop_inventory_alpha_mask = 29
tableau_troop_inventory_color = 30
tableau_troop_profile_alpha_mask = 31
tableau_troop_profile_color = 32
tableau_troop_party_alpha_mask = 33
tableau_troop_party_color = 34
tableau_troop_note_mesh = 35
tableau_center_note_mesh = 36
tableau_faction_note_mesh_for_menu = 37
tableau_faction_note_mesh = 38
tableau_faction_note_mesh_banner = 39
tableau_2_factions_mesh = 40
tableau_color_picker = 41
tableau_custom_banner_square_no_mesh = 42
tableau_custom_banner_default = 43
tableau_custom_banner_tall = 44
tableau_custom_banner_square = 45
tableau_custom_banner_short = 46
tableau_background_selection = 47
tableau_positioning_selection = 48
tableau_retired_troop_alpha_mask = 49
tableau_retired_troop_color = 50
tableau_retirement_troop = 51



from smart_module.module_system.module_items import *
from smart_module.module_system.process_common import *
from smart_module.module_system.process_operations import *


def get_item_code(item):
  prefix = "it_"
  code = prefix + item[0]
  return code

def save_python_header(src_dir):
  file = open(src_dir + "ID_items.py","w",encoding="utf-8")
  for i_item in range(len(items)):
    file.write("itm_%s = %d\n"%(convert_to_identifier(items[i_item][0]),i_item))
  file.close()

def write_items(variable_list,variable_uses,tag_uses,quick_strings,export_dir):
  itemkinds_file_name = export_dir + "item_kinds1.txt"
  ofile = open(itemkinds_file_name,"w",encoding="utf-8")
  ofile.write("itemsfile version 3\n")
  ofile.write("%d\n"%len(items))
  for item in items:
    if (item[3] & itp_merchandise) > 0:
      id_no = find_object("itm",convert_to_identifier(item[0]))
      add_tag_use(tag_uses,tag_item,id_no)
    ofile.write(" itm_%s %s %s %d "%(convert_to_identifier(item[0]),replace_spaces(item[1]),replace_spaces(item[1]),len(item[2])))
    item_variations = item[2]
    for item_variation in item_variations:
      ofile.write(" %s %d "%(item_variation[0],item_variation[1]))
    ofile.write(" %d %d %d %d %f %d %d %d %d %d %d %d %d %d %d %d %d\n"%(item[3], item[4], item[5], item[7],
                                                   get_weight(item[6]),
                                                   get_abundance(item[6]),                  
                                                   get_head_armor(item[6]),
                                                   get_body_armor(item[6]),
                                                   get_leg_armor(item[6]),
                                                   get_difficulty(item[6]),
                                                   get_hit_points(item[6]),
                                                   get_speed_rating(item[6]),
                                                   get_missile_speed(item[6]),
                                                   get_weapon_length(item[6]),
                                                   get_max_ammo(item[6]),
                                                   get_thrust_damage(item[6]),
                                                   get_swing_damage(item[6]),
                                                               ))
    if (len(item) > 9):
      ofile.write(" %d\n"%(len(item[9])))
      for item_faction in item[9]:
        ofile.write(" %d"%item_faction)
      ofile.write("\n")
    else:
      ofile.write(" 0\n")
    trigger_list = []
    if (len(item) > 8):
      trigger_list = item[8]
    save_simple_triggers(ofile,trigger_list, variable_list,variable_uses,tag_uses,quick_strings)


  ofile.close()


def processItems(context):
    print( "Exporting item data...")

    configParser = context.configParser
    src_dir = configParser.getSrcDir()
    export_dir = configParser.getExportDir()

    save_python_header(src_dir)


    variable_uses = []
    variables = load_variables(export_dir,variable_uses)
    tag_uses = load_tag_uses(export_dir)
    quick_strings = load_quick_strings(export_dir)
    write_items(variables,variable_uses,tag_uses,quick_strings,export_dir)
    save_variables(export_dir,variables,variable_uses)
    save_tag_uses(export_dir,tag_uses)
    save_quick_strings(export_dir,quick_strings)
    #print( "Finished with Items.")



import os

from smart_module.smart_core.smart_modules_log import info, debug


################################################
###
### 汉化器：汉化工作
###  1.先使用旧逻辑,加快使用速度
###  2.以后优化建议：
###     1.汉化文件单独一个文件夹（只能读取不能修改）
###     2.汉化时读取文件内容，汉化smart module模块的汉化数据，最后生成最终汉化的目录
###     3.汉化目录的结构和剧本保持一致，以便生成补丁
###
################################################




class SmartModuleLanguageExecutor:
    "这是一个SmartModule功能错误信息 "
    def __init__(self,context):
        configParser = context.configParser
        src_dir = configParser.getSrcDir()
        export_dir = configParser.getExportDir()
        ## 保证输出目录存在
        if not os.path.exists(export_dir):
            print(os.path.abspath(export_dir))
            os.mkdir(export_dir)

        export_dir = configParser.getExportDir()
        self.languageDatas = {}
        self.path = src_dir + "languages/"
        self.outputPath = export_dir + "languages/"
        ## 保存汉化目录存在
        if not os.path.exists(self.outputPath):
            print(os.path.abspath(self.outputPath))
            os.mkdir(self.outputPath)
    def __str__(self):
        return "smart module 汉化器"

    def parseLanguages(self):
        languages = os.listdir(self.path)
        for lanuage in languages:
            lanuageDir = os.path.join(self.path,lanuage)
            if os.path.isdir(lanuageDir):
                fileDatas = {}
                files = os.listdir(lanuageDir)
                for fileName in files:
                    filePath = os.path.join(lanuageDir, fileName)
                    if os.path.isfile(filePath) and filePath.endswith(".csv"):
                        lines = {}
                        ## 去掉后缀
                        fileKey = fileName[:-4]

                        file = open(filePath, "r", encoding="UTF-8-sig")
                        fileLines = file.readlines()
                        file.close()
                        for line in fileLines:
                            if len(line.strip()) > 0:
                                lineDatas = line.split("|")
                                lines[lineDatas[0]] = lineDatas[1]
                        fileDatas[fileKey] = lines
                self.languageDatas[lanuage] = fileDatas

    def parseModules(self,modules):
        moduleDatas = {}
        for module in modules:
            if "internationals" in module:
                fileInfo = module.get("internationals")
                for (fold, files) in fileInfo.items():
                    foldDatas = moduleDatas.get(fold)
                    if foldDatas == None:
                        foldDatas = {}
                        moduleDatas[fold] = foldDatas
                    for (filename, infos) in files.items():
                        fileDatas = foldDatas.get(filename)
                        if fileDatas == None:
                            fileDatas = []
                            foldDatas[filename] = fileDatas
                        lines = []
                        for line in infos:
                            lines.append(line + "\n")
                        fileDatas.extend(lines)
        return moduleDatas

    def mergeLanguageDatas(self,moduleLanguages):
        for language,datas in moduleLanguages.items():
            moduleData = self.languageDatas.get(language)
            for filename,lines in datas.items():
                filedata = moduleData.get(filename)
                for line in lines:
                    lineDatas = line.split("|")
                    filedata[lineDatas[0]] = lineDatas[1]

    def writeDataToFile(self):
        for language,files in self.languageDatas.items():
            dirPath = os.path.join(self.outputPath,language)
            if not os.path.exists(dirPath):
                os.mkdir(dirPath)
            for filename,fileData in files.items():
                lines = []
                for key,val in fileData.items():
                    if filename == "quick_strings":
                        ## 快捷字符符汉化时左边标识符最多只支持25个字符
                        lines.append(key[0:25] + "|" + val)
                    else:
                        lines.append(key + "|" + val)
                ## 写入文件
                filePath = os.path.join(dirPath,filename + ".csv")
                file = open(filePath, "w", encoding="utf-8")
                file.writelines(lines)
                file.close()

    def preprocessInternational(self,modules):
        '''
            处理汉化功能
            将汉化信息写入到导出目录的languages对应的汉化文件夹中的文件里
        :return:
        '''
        info("【汉化】smartModule所有模块")
        self.parseLanguages()
        debug("【汉化】解析languages文件完成")
        moduleLanguages = self.parseModules(modules)
        debug("【汉化】解析modules模块完成")
        self.mergeLanguageDatas(moduleLanguages)
        debug("【汉化】合并modules数据完成")
        self.writeDataToFile()
        info("【汉化】完成")


# if __name__ == '__main__':
#     # executor = SmartModuleLanguageExecutor()
#     # executor.preprocessInternational(modules)



from smart_module.module_system.module_info import *
from smart_module.module_system.module_particle_systems import *

id_pos        = 0
flags_pos     = 1
mesh_name_pos = 2
num_particles_pos = 3
life_pos      = 4
damping_pos   = 5
gravity_pos   = 6
turb_size_pos = 7
turb_wt_pos   = 8
alpha_key_pos = 9
red_key_pos         = alpha_key_pos + 2
green_key_pos       = red_key_pos   + 2
blue_key_pos        = green_key_pos + 2
scale_key_pos       = blue_key_pos  + 2
emit_box_size_pos   = scale_key_pos + 2
emit_velocity_pos   = emit_box_size_pos + 1
emit_rndmness_pos   = emit_velocity_pos + 1
angular_speed_pos   = emit_rndmness_pos + 1
angular_damping_pos = angular_speed_pos + 1

def save_psys_keys(ofile, keys1, keys2):
    ofile.write("%f %f   %f %f\n"%(keys1[0], keys1[1], keys2[0], keys2[1]))

def save_particle_systems(export_dir):
  ofile = open(export_dir + "particle_systems.txt","w",encoding="utf-8")
  ofile.write("particle_systemsfile version 1\n")
  ofile.write("%d\n"%len(particle_systems))
  for psys in particle_systems:
    ofile.write("psys_%s %d %s  "%(psys[0], psys[1], psys[2]))
    ofile.write("%d %f %f %f %f %f \n"%(psys[num_particles_pos], psys[life_pos], psys[damping_pos], psys[gravity_pos], psys[turb_size_pos], psys[turb_wt_pos]))
    save_psys_keys(ofile,psys[alpha_key_pos],psys[alpha_key_pos+1])
    save_psys_keys(ofile,psys[red_key_pos],psys[red_key_pos+1])
    save_psys_keys(ofile,psys[green_key_pos],psys[green_key_pos+1])
    save_psys_keys(ofile,psys[blue_key_pos],psys[blue_key_pos+1])
    save_psys_keys(ofile,psys[scale_key_pos],psys[scale_key_pos+1])
    ofile.write("%f %f %f   "%(psys[emit_box_size_pos][0],psys[emit_box_size_pos][1],psys[emit_box_size_pos][2]))
    ofile.write("%f %f %f   "%(psys[emit_velocity_pos][0],psys[emit_velocity_pos][1],psys[emit_velocity_pos][2]))
    ofile.write("%f \n"%(psys[emit_rndmness_pos]))
    if (len(psys) >= (angular_speed_pos + 1)):
      ofile.write("%f "%(psys[angular_speed_pos]))
    else:
      ofile.write("0.0 ")
    if (len(psys) >= (angular_damping_pos + 1)):
      ofile.write("%f "%(psys[angular_damping_pos]))
    else:
      ofile.write("0.0 ")
    ofile.write("\n")
  ofile.close()

def save_python_header(src_dir):
  ofile = open(src_dir + "ID_particle_systems.py","w",encoding="utf-8")
  for i_particle_system in range(len(particle_systems)):
    ofile.write("psys_%s = %d\n"%(particle_systems[i_particle_system][0],i_particle_system))
  ofile.close()


def processParticleSys(context):
  print( "Exporting particle data...")

  configParser = context.configParser
  src_dir = configParser.getSrcDir()
  export_dir = configParser.getExportDir()

  save_particle_systems(export_dir)
  save_python_header(src_dir)



from smart_module.module_system.header_common import *

###################################################
# header_troops.py
# This file contains declarations for troops
# DO NOT EDIT THIS FILE!
###################################################

#Troop flags
tf_male           = 0
tf_female         = 1
tf_undead         = 2
troop_type_mask   = 0x0000000f
tf_hero              = 0x00000010
tf_inactive          = 0x00000020
tf_unkillable        = 0x00000040
tf_allways_fall_dead = 0x00000080
tf_no_capture_alive  = 0x00000100
tf_mounted           = 0x00000400 #Troop's movement speed on map is determined by riding skill.
tf_is_merchant       = 0x00001000 #When set, troop does not equip stuff he owns
tf_randomize_face    = 0x00008000 #randomize face at the beginning of the game.

tf_guarantee_boots            = 0x00100000
tf_guarantee_armor            = 0x00200000
tf_guarantee_helmet           = 0x00400000
tf_guarantee_gloves           = 0x00800000
tf_guarantee_horse            = 0x01000000
tf_guarantee_shield           = 0x02000000
tf_guarantee_ranged           = 0x04000000
tf_unmoveable_in_party_window = 0x10000000

# Character attributes...
ca_strength     = 0
ca_agility      = 1
ca_intelligence = 2
ca_charisma     = 3

wpt_one_handed_weapon = 0
wpt_two_handed_weapon = 1
wpt_polearm           = 2
wpt_archery           = 3
wpt_crossbow          = 4
wpt_throwing          = 5
wpt_firearm           = 6


#personality modifiers:
# courage 8 means neutral
courage_4  = 0x0004
courage_5  = 0x0005
courage_6  = 0x0006
courage_7  = 0x0007
courage_8  = 0x0008
courage_9  = 0x0009
courage_10 = 0x000A
courage_11 = 0x000B
courage_12 = 0x000C
courage_13 = 0x000D
courage_14 = 0x000E
courage_15 = 0x000F

aggresiveness_1  = 0x0010
aggresiveness_2  = 0x0020
aggresiveness_3  = 0x0030
aggresiveness_4  = 0x0040
aggresiveness_5  = 0x0050
aggresiveness_6  = 0x0060
aggresiveness_7  = 0x0070
aggresiveness_8  = 0x0080
aggresiveness_9  = 0x0090
aggresiveness_10 = 0x00A0
aggresiveness_11 = 0x00B0
aggresiveness_12 = 0x00C0
aggresiveness_13 = 0x00D0
aggresiveness_14 = 0x00E0
aggresiveness_15 = 0x00F0

is_bandit        = 0x0100
#-----------------------------------
#these also in sentences.py
tsf_site_id_mask = 0x0000ffff
tsf_entry_mask   = 0x00ff0000
tsf_entry_bits   = 16

def entry(n):
  return (((n) << tsf_entry_bits) & tsf_entry_mask)
#-------------------------------------

str_3            = bignum | 0x00000003
str_4            = bignum | 0x00000004
str_5            = bignum | 0x00000005
str_6            = bignum | 0x00000006
str_7            = bignum | 0x00000007
str_8            = bignum | 0x00000008
str_9            = bignum | 0x00000009
str_10           = bignum | 0x0000000a
str_11           = bignum | 0x0000000b
str_12           = bignum | 0x0000000c
str_13           = bignum | 0x0000000d
str_14           = bignum | 0x0000000e
str_15           = bignum | 0x0000000f
str_16           = bignum | 0x00000010
str_17           = bignum | 0x00000011
str_18           = bignum | 0x00000012
str_19           = bignum | 0x00000013
str_20           = bignum | 0x00000014
str_21           = bignum | 0x00000015
str_22           = bignum | 0x00000016
str_23           = bignum | 0x00000017
str_24           = bignum | 0x00000018
str_25           = bignum | 0x00000019
str_26           = bignum | 0x0000001a
str_27           = bignum | 0x0000001b
str_28           = bignum | 0x0000001c
str_29           = bignum | 0x0000001d
str_30           = bignum | 0x0000001e

agi_3            = bignum | 0x00000300
agi_4            = bignum | 0x00000400
agi_5            = bignum | 0x00000500
agi_6            = bignum | 0x00000600
agi_7            = bignum | 0x00000700
agi_8            = bignum | 0x00000800
agi_9            = bignum | 0x00000900
agi_10           = bignum | 0x00000a00
agi_11           = bignum | 0x00000b00
agi_12           = bignum | 0x00000c00
agi_13           = bignum | 0x00000d00
agi_14           = bignum | 0x00000e00
agi_15           = bignum | 0x00000f00
agi_16           = bignum | 0x00001000
agi_17           = bignum | 0x00001100
agi_18           = bignum | 0x00001200
agi_19           = bignum | 0x00001300
agi_20           = bignum | 0x00001400
agi_21           = bignum | 0x00001500
agi_22           = bignum | 0x00001600
agi_23           = bignum | 0x00001700
agi_24           = bignum | 0x00001800
agi_25           = bignum | 0x00001900
agi_26           = bignum | 0x00001a00
agi_27           = bignum | 0x00001b00
agi_28           = bignum | 0x00001c00
agi_29           = bignum | 0x00001d00
agi_30           = bignum | 0x00001e00

int_3            = bignum | 0x00030000
int_4            = bignum | 0x00040000
int_5            = bignum | 0x00050000
int_6            = bignum | 0x00060000
int_7            = bignum | 0x00070000
int_8            = bignum | 0x00080000
int_9            = bignum | 0x00090000
int_10           = bignum | 0x000a0000
int_11           = bignum | 0x000b0000
int_12           = bignum | 0x000c0000
int_13           = bignum | 0x000d0000
int_14           = bignum | 0x000e0000
int_15           = bignum | 0x000f0000
int_16           = bignum | 0x00100000
int_17           = bignum | 0x00110000
int_18           = bignum | 0x00120000
int_19           = bignum | 0x00130000
int_20           = bignum | 0x00140000
int_21           = bignum | 0x00150000
int_22           = bignum | 0x00160000
int_23           = bignum | 0x00170000
int_24           = bignum | 0x00180000
int_25           = bignum | 0x00190000
int_26           = bignum | 0x001a0000
int_27           = bignum | 0x001b0000
int_28           = bignum | 0x001c0000
int_29           = bignum | 0x001d0000
int_30           = bignum | 0x001e0000


cha_3            = bignum | 0x03000000
cha_4            = bignum | 0x04000000
cha_5            = bignum | 0x05000000
cha_6            = bignum | 0x06000000
cha_7            = bignum | 0x07000000
cha_8            = bignum | 0x08000000
cha_9            = bignum | 0x09000000
cha_10           = bignum | 0x0a000000
cha_11           = bignum | 0x0b000000
cha_12           = bignum | 0x0c000000
cha_13           = bignum | 0x0d000000
cha_14           = bignum | 0x0e000000
cha_15           = bignum | 0x0f000000
cha_16           = bignum | 0x10000000
cha_17           = bignum | 0x11000000
cha_18           = bignum | 0x12000000
cha_19           = bignum | 0x13000000
cha_20           = bignum | 0x14000000
cha_21           = bignum | 0x15000000
cha_22           = bignum | 0x16000000
cha_23           = bignum | 0x17000000
cha_24           = bignum | 0x18000000
cha_25           = bignum | 0x19000000
cha_26           = bignum | 0x1a000000
cha_27           = bignum | 0x1b000000
cha_28           = bignum | 0x1c000000
cha_29           = bignum | 0x1d000000
cha_30           = bignum | 0x1e000000

level_mask       = 0x000000FF
level_bits       = 32

def level(v):
  if (v > level_mask):
    v = level_mask
  return (bignum|v) << level_bits
  
def_attrib = str_5 | agi_5 | int_4 | cha_4

# Weapon proficiencies:
one_handed_bits = 0
two_handed_bits = 10
polearm_bits    = 20
archery_bits    = 30
crossbow_bits   = 40
throwing_bits   = 50
firearm_bits    = 60

num_weapon_proficiencies = 7
def wp_one_handed(x):
  return (((bignum | x) & 0x3FF) << one_handed_bits)
def wp_two_handed(x):
  return (((bignum | x) & 0x3FF) << two_handed_bits)
def wp_polearm(x):
  return (((bignum | x) & 0x3FF) << polearm_bits)
def wp_archery(x):
  return (((bignum | x) & 0x3FF) << archery_bits)
def wp_crossbow(x):
  return (((bignum | x) & 0x3FF) << crossbow_bits)
def wp_throwing(x):
  return (((bignum | x) & 0x3FF) << throwing_bits)
def wp_firearm(x):
  return (((bignum | x) & 0x3FF) << firearm_bits)

def find_troop(troops,troop_id):
  result = -1
  num_troops = len(troops)
  i_troop = 0
  while (i_troop < num_troops) and (result == -1):
    troop = troops[i_troop]
    if (troop[0] == troop_id):
      result = i_troop
    else:
      i_troop += 1
  return result



def upgrade(troops,troop1_id,troop2_id):
  troop1_no = find_troop(troops,troop1_id)
  troop2_no = find_troop(troops,troop2_id)
  if (troop1_no == -1):
    print( "Error with upgrade def: Unable to find troop1-id: " + troop1_id)
  elif (troop2_no == -1):
    print( "Error with upgrade def: Unable to find troop2-id: " + troop2_id)
  else:
    cur_troop = troops[troop1_no]
    cur_troop_length = len(cur_troop)
    if cur_troop_length == 11:
      cur_troop[11:11] = [0, 0, 0, troop2_no, 0]
    elif cur_troop_length == 12:
      cur_troop[12:12] = [0, 0, troop2_no, 0]
    elif cur_troop_length == 13:
      cur_troop[13:13] = [0, troop2_no, 0]
    else:
      cur_troop[14:14] = [troop2_no, 0]
      

def upgrade2(troops,troop1_id,troop2_id,troop3_id):
  troop1_no = find_troop(troops,troop1_id)
  troop2_no = find_troop(troops,troop2_id)
  troop3_no = find_troop(troops,troop3_id)
  if (troop1_no == -1):
    print( "Error with upgrade2 def: Unable to find troop1-id: " + troop1_id)
  elif (troop2_no == -1):
    print( "Error with upgrade2 def: Unable to find troop2-id: " + troop2_id)
  elif (troop3_no == -1):
    print( "Error with upgrade2 def: Unable to find troop3-id: " + troop3_id)
  else:
    cur_troop = troops[troop1_no]
    cur_troop_length = len(cur_troop)
    if cur_troop_length == 11:
      cur_troop[11:11] = [0, 0, 0, troop2_no, troop3_no]
    elif cur_troop_length == 12:
      cur_troop[12:12] = [0, 0, troop2_no, troop3_no]
    elif cur_troop_length == 13:
      cur_troop[13:13] = [0, troop2_no, troop3_no]
    else:
      cur_troop[14:14] = [troop2_no, troop3_no]



###################################################
# header items.py
# This file contains declarations for items
# DO NOT EDIT THIS FILE!
###################################################



from smart_module.module_system.header_common import *
from smart_module.module_system.header_item_modifiers import *

#item flags
itp_type_horse           = 0x0000000000000001
itp_type_one_handed_wpn  = 0x0000000000000002
itp_type_two_handed_wpn  = 0x0000000000000003
itp_type_polearm         = 0x0000000000000004
itp_type_arrows          = 0x0000000000000005
itp_type_bolts           = 0x0000000000000006
itp_type_shield          = 0x0000000000000007
itp_type_bow             = 0x0000000000000008
itp_type_crossbow        = 0x0000000000000009
itp_type_thrown          = 0x000000000000000a
itp_type_goods           = 0x000000000000000b
itp_type_head_armor      = 0x000000000000000c
itp_type_body_armor      = 0x000000000000000d
itp_type_foot_armor      = 0x000000000000000e
itp_type_hand_armor      = 0x000000000000000f
itp_type_pistol          = 0x0000000000000010
itp_type_musket          = 0x0000000000000011
itp_type_bullets         = 0x0000000000000012
itp_type_animal          = 0x0000000000000013
itp_type_book            = 0x0000000000000014

##itp_attach_left_item     = 0x000000100
##itp_attach_left_hand     = 0x000000200
##itp_attach_forearm       = 0x000000300
##itp_attach_armature      = 0x000000f00

itp_force_attach_left_hand      = 0x0000000000000100
itp_force_attach_right_hand     = 0x0000000000000200
itp_force_attach_left_forearm   = 0x0000000000000300
itp_attach_armature             = 0x0000000000000f00
itp_attachment_mask             = 0x0000000000000f00


itp_unique               = 0x0000000000001000
itp_always_loot          = 0x0000000000002000
##itp_melee                = 0x000002000
itp_no_parry             = 0x0000000000004000
##itp_spear                = 0x000008000 #obsolete
itp_default_ammo         = 0x0000000000008000
itp_merchandise          = 0x0000000000010000
itp_wooden_attack        = 0x0000000000020000
itp_wooden_parry         = 0x0000000000040000
itp_food                 = 0x0000000000080000
#itp_bow                  = 0x0000000000000010
#itp_shield               = 0x0000000000000020
#itp_javelin              = 0x0000000000000040
#itp_crossbow             = 0x0000000000000080

itp_cant_reload_on_horseback = 0x0000000000100000
itp_two_handed               = 0x0000000000200000
itp_primary                  = 0x0000000000400000
itp_secondary                = 0x0000000000800000
itp_covers_legs              = 0x0000000001000000
itp_doesnt_cover_hair        = 0x0000000001000000
itp_can_penetrate_shield     = 0x0000000001000000
itp_consumable               = 0x0000000002000000
itp_bonus_against_shield     = 0x0000000004000000
itp_penalty_with_shield      = 0x0000000008000000
itp_cant_use_on_horseback    = 0x0000000010000000
itp_civilian                 = 0x0000000020000000
itp_next_item_as_melee       = 0x0000000020000000
itp_fit_to_head              = 0x0000000040000000
itp_offset_lance             = 0x0000000040000000
itp_covers_head              = 0x0000000080000000
itp_couchable                = 0x0000000080000000
itp_crush_through            = 0x0000000100000000
#itp_knock_back               = 0x0000000200000000 being used?
itp_remove_item_on_use       = 0x0000000400000000
itp_unbalanced               = 0x0000000800000000

itp_covers_beard             = 0x0000001000000000    #remove beard mesh
itp_no_pick_up_from_ground   = 0x0000002000000000
itp_can_knock_down           = 0x0000004000000000
itp_extra_penetration        = 0x0000100000000000
itp_has_bayonet              = 0x0000200000000000
itp_cant_reload_while_moving = 0x0000400000000000
itp_ignore_gravity           = 0x0000800000000000
itp_ignore_friction          = 0x0001000000000000
itp_is_pike                  = 0x0002000000000000
itp_offset_musket            = 0x0004000000000000
itp_no_blur                  = 0x0008000000000000

itp_cant_reload_while_moving_mounted = 0x0010000000000000
itp_has_upper_stab           = 0x0020000000000000
itp_kill_info_mask           = 0x0700000000000000
itp_kill_info_bits           = 56 # 0x0700000000000000

#equipment slots
ek_item_0 = 0
ek_item_1 = 1
ek_item_2 = 2
ek_item_3 = 3
ek_head   = 4
ek_body   = 5
ek_foot   = 6
ek_gloves = 7
ek_horse  = 8
ek_food   = 9


max_inventory_items = 96
num_equipment_kinds = ek_food + 1
num_weapon_proficiencies = 7

#damage types:
cut    = 0
pierce = 1
blunt  = 2


ibf_armor_mask           = 0x00000000000000000000000ff
ibf_damage_mask          = 0x00000000000000000000003ff
ibf_10bit_mask           = 0x00000000000000000000003ff

ibf_head_armor_bits      = 0
ibf_body_armor_bits      = 8
ibf_leg_armor_bits       = 16
ibf_weight_bits          = 24
ibf_difficulty_bits      = 32

ibf_hitpoints_mask       = 0x0000ffff
ibf_hitpoints_bits       = 40

#iwf_damage_mask             = 0x10000000000000ff #make sure value is 64 bits so that << will work
iwf_swing_damage_bits       = 50
iwf_swing_damage_type_bits  = 58
iwf_thrust_damage_bits      = 60
iwf_thrust_damage_type_bits = 68
iwf_weapon_length_bits      = 70
iwf_speed_rating_bits       = 80
iwf_shoot_speed_bits        = 90

iwf_max_ammo_bits           = 100 # use this for shield endurance too?
iwf_abundance_bits          = 110
iwf_accuracy_bits           = 16  #reuse leg_armor for accuracy  
#iwf_horse_speed_bits        = 8
#iwf_horse_maneuver_bits     = 16
#iwf_horse_charge_bits       = 24

iwf_damage_type_bits = 8


def weight(x):
  a = int(4.0 * x)
  a = ibf_armor_mask & a
  return (((bignum | a) & ibf_armor_mask) << ibf_weight_bits)
  
def get_weight(y):
  a = (y >> ibf_weight_bits) & ibf_armor_mask
  return 0.25 * a 

def head_armor(x):
  return (((bignum | x) & ibf_armor_mask) << ibf_head_armor_bits)

def get_head_armor(y):
  return (y >> ibf_head_armor_bits) & ibf_armor_mask

def body_armor(x):
  return (((bignum | x) & ibf_armor_mask) << ibf_body_armor_bits)

def get_body_armor(y):
  return (y >> ibf_body_armor_bits) & ibf_armor_mask

def leg_armor(x):
  return (((bignum | x) & ibf_armor_mask) << ibf_leg_armor_bits)

def get_leg_armor(y):
  return (y >> ibf_leg_armor_bits) & ibf_armor_mask

def difficulty(x):
  return (((bignum | x) & ibf_armor_mask) << ibf_difficulty_bits)

def get_difficulty(y):
  return (y >> ibf_difficulty_bits) & ibf_armor_mask

def hit_points(x):
  return (((bignum | x) & ibf_hitpoints_mask) << ibf_hitpoints_bits)

def get_hit_points(y):
  return (y >> ibf_hitpoints_bits) & ibf_hitpoints_mask

def spd_rtng(x):
  return (((bignum | x) & ibf_armor_mask) << iwf_speed_rating_bits)

def get_speed_rating(y):
  return (y >> iwf_speed_rating_bits) & ibf_armor_mask

def shoot_speed(x):
  return (((bignum | x) & ibf_10bit_mask) << iwf_shoot_speed_bits)

def get_missile_speed(y):
  return (y >> iwf_shoot_speed_bits) & ibf_10bit_mask

def horse_scale(x):
  return (((bignum | x) & ibf_10bit_mask) << iwf_weapon_length_bits)

def weapon_length(x):
  return (((bignum | x) & ibf_10bit_mask) << iwf_weapon_length_bits)

def shield_width(x):
  return weapon_length(x)
 
def shield_height(x):
  return shoot_speed(x)
 
def get_weapon_length(y):
  return ((y >> iwf_weapon_length_bits) & ibf_10bit_mask)

def max_ammo(x):
  return (((bignum | x) & ibf_armor_mask) << iwf_max_ammo_bits)

def get_max_ammo(y):
  return (y >> iwf_max_ammo_bits) & ibf_armor_mask

def swing_damage(damage,damage_type):
  x = ((damage_type << iwf_damage_type_bits) | (damage & ibf_armor_mask))
  return (((bignum | x) & ibf_damage_mask) << iwf_swing_damage_bits)

def get_swing_damage(y):
  return (y >> iwf_swing_damage_bits) & ibf_damage_mask

def thrust_damage(damage,damage_type):
  x = ((damage_type << iwf_damage_type_bits) | (damage & ibf_armor_mask))
  return (((bignum | x) & ibf_damage_mask) << iwf_thrust_damage_bits)

def get_thrust_damage(y):
  return (y >> iwf_thrust_damage_bits) & ibf_damage_mask

def horse_speed(x):
  return shoot_speed(x)

def horse_maneuver(x):
  return spd_rtng(x)

def horse_charge(x):
  return thrust_damage(x,0)

def food_quality(x):
  return head_armor(x)

def abundance(x):
  return (((bignum | x) & ibf_armor_mask) << iwf_abundance_bits)

def get_abundance(y):
  abnd = (y >> iwf_abundance_bits) & ibf_armor_mask
  if (abnd == 0):
    abnd = 100
  return abnd

def accuracy(x):
  return leg_armor(x)

def custom_kill_info(x): # you have to add ico_custom_x (where x is a number between 1 and 7) mesh in order to display it correctly.
  return (((bignum | x) & (itp_kill_info_mask >> itp_kill_info_bits)) << itp_kill_info_bits)

# Item capabilities:
itcf_thrust_onehanded                                = 0x0000000000000001
itcf_overswing_onehanded                             = 0x0000000000000002
itcf_slashright_onehanded                            = 0x0000000000000004
itcf_slashleft_onehanded                             = 0x0000000000000008

itcf_thrust_twohanded                                = 0x0000000000000010
itcf_overswing_twohanded                             = 0x0000000000000020
itcf_slashright_twohanded                            = 0x0000000000000040
itcf_slashleft_twohanded                             = 0x0000000000000080

itcf_thrust_polearm                                  = 0x0000000000000100
itcf_overswing_polearm                               = 0x0000000000000200
itcf_slashright_polearm                              = 0x0000000000000400
itcf_slashleft_polearm                               = 0x0000000000000800

itcf_shoot_bow                                       = 0x0000000000001000
itcf_shoot_javelin                                   = 0x0000000000002000
itcf_shoot_crossbow                                  = 0x0000000000004000

itcf_throw_stone                                     = 0x0000000000010000
itcf_throw_knife                                     = 0x0000000000020000
itcf_throw_axe                                       = 0x0000000000030000
itcf_throw_javelin                                   = 0x0000000000040000
itcf_shoot_pistol                                    = 0x0000000000070000
itcf_shoot_musket                                    = 0x0000000000080000
itcf_shoot_mask                                      = 0x00000000000ff000

itcf_horseback_thrust_onehanded                      = 0x0000000000100000
itcf_horseback_overswing_right_onehanded             = 0x0000000000200000
itcf_horseback_overswing_left_onehanded              = 0x0000000000400000
itcf_horseback_slashright_onehanded                  = 0x0000000000800000
itcf_horseback_slashleft_onehanded                   = 0x0000000001000000
itcf_thrust_onehanded_lance                          = 0x0000000004000000
itcf_thrust_onehanded_lance_horseback                = 0x0000000008000000

itcf_carry_mask                                      = 0x00000007f0000000
itcf_carry_sword_left_hip                            = 0x0000000010000000
itcf_carry_axe_left_hip                              = 0x0000000020000000
itcf_carry_dagger_front_left                         = 0x0000000030000000
itcf_carry_dagger_front_right                        = 0x0000000040000000
itcf_carry_quiver_front_right                        = 0x0000000050000000
itcf_carry_quiver_back_right                         = 0x0000000060000000
itcf_carry_quiver_right_vertical                     = 0x0000000070000000
itcf_carry_quiver_back                               = 0x0000000080000000
itcf_carry_revolver_right                            = 0x0000000090000000
itcf_carry_pistol_front_left                         = 0x00000000a0000000
itcf_carry_bowcase_left                              = 0x00000000b0000000
itcf_carry_mace_left_hip                             = 0x00000000c0000000

itcf_carry_axe_back                                  = 0x0000000100000000
itcf_carry_sword_back                                = 0x0000000110000000
itcf_carry_kite_shield                               = 0x0000000120000000
itcf_carry_round_shield                              = 0x0000000130000000
itcf_carry_buckler_left                              = 0x0000000140000000
itcf_carry_crossbow_back                             = 0x0000000150000000
itcf_carry_bow_back                                  = 0x0000000160000000
itcf_carry_spear                                     = 0x0000000170000000
itcf_carry_board_shield                              = 0x0000000180000000

itcf_carry_katana                                    = 0x0000000210000000
itcf_carry_wakizashi                                 = 0x0000000220000000


itcf_show_holster_when_drawn                         = 0x0000000800000000

itcf_reload_pistol                                   = 0x0000007000000000
itcf_reload_musket                                   = 0x0000008000000000
itcf_reload_mask                                     = 0x000000f000000000

itcf_parry_forward_onehanded                         = 0x0000010000000000
itcf_parry_up_onehanded                              = 0x0000020000000000
itcf_parry_right_onehanded                           = 0x0000040000000000
itcf_parry_left_onehanded                            = 0x0000080000000000

itcf_parry_forward_twohanded                         = 0x0000100000000000
itcf_parry_up_twohanded                              = 0x0000200000000000
itcf_parry_right_twohanded                           = 0x0000400000000000
itcf_parry_left_twohanded                            = 0x0000800000000000

itcf_parry_forward_polearm                           = 0x0001000000000000
itcf_parry_up_polearm                                = 0x0002000000000000
itcf_parry_right_polearm                             = 0x0004000000000000
itcf_parry_left_polearm                              = 0x0008000000000000

itcf_horseback_slash_polearm                         = 0x0010000000000000
itcf_overswing_spear                                 = 0x0020000000000000
itcf_overswing_musket                                = 0x0040000000000000
itcf_thrust_musket                                   = 0x0080000000000000

itcf_force_64_bits                                   = 0x8000000000000000

#combined capabilities
itc_cleaver = itcf_force_64_bits | (itcf_overswing_onehanded|itcf_slashright_onehanded|itcf_slashleft_onehanded |
                                    itcf_horseback_slashright_onehanded|itcf_horseback_slashleft_onehanded)
itc_dagger  = itc_cleaver | itcf_thrust_onehanded

itc_parry_onehanded = itcf_force_64_bits | itcf_parry_forward_onehanded| itcf_parry_up_onehanded | itcf_parry_right_onehanded |itcf_parry_left_onehanded
itc_longsword = itc_dagger | itc_parry_onehanded
itc_scimitar  = itc_cleaver | itc_parry_onehanded

itc_parry_two_handed = itcf_force_64_bits | itcf_parry_forward_twohanded | itcf_parry_up_twohanded | itcf_parry_right_twohanded | itcf_parry_left_twohanded
itc_cut_two_handed = itcf_force_64_bits | (itcf_slashright_twohanded | itcf_slashleft_twohanded | itcf_overswing_twohanded | 
                                           itcf_horseback_slashright_onehanded|itcf_horseback_slashleft_onehanded)
itc_greatsword = itc_cut_two_handed |  itcf_thrust_twohanded | itc_parry_two_handed |itcf_thrust_onehanded_lance
itc_nodachi    = itc_cut_two_handed | itc_parry_two_handed

itc_bastardsword = itc_cut_two_handed |  itcf_thrust_twohanded | itc_parry_two_handed |itc_dagger
itc_morningstar = itc_cut_two_handed |  itc_parry_two_handed |itc_cleaver

itc_parry_polearm = itcf_parry_forward_polearm | itcf_parry_up_polearm | itcf_parry_right_polearm | itcf_parry_left_polearm
itc_poleaxe    = itc_parry_polearm| itcf_overswing_polearm |itcf_thrust_polearm|itcf_slashright_polearm|itcf_slashleft_polearm
itc_staff      = itc_parry_polearm| itcf_thrust_onehanded_lance |itcf_thrust_onehanded_lance_horseback| itcf_overswing_polearm |itcf_thrust_polearm|itcf_slashright_polearm|itcf_slashleft_polearm
itc_spear      = itc_parry_polearm| itcf_thrust_onehanded_lance |itcf_thrust_onehanded_lance_horseback | itcf_thrust_polearm
itc_cutting_spear = itc_spear|itcf_overswing_polearm
itc_pike       = itcf_thrust_onehanded_lance |itcf_thrust_onehanded_lance_horseback | itcf_thrust_polearm
itc_guandao    = itc_parry_polearm|itcf_overswing_polearm|itcf_thrust_polearm|itcf_slashright_polearm|itcf_slashleft_polearm|itcf_horseback_slashright_onehanded|itcf_horseback_slashleft_onehanded|itcf_horseback_slash_polearm

itc_greatlance = itcf_thrust_onehanded_lance |itcf_thrust_onehanded_lance_horseback| itcf_thrust_polearm
itc_musket_melee = itc_parry_polearm|itcf_overswing_musket|itcf_thrust_musket|itcf_slashright_twohanded|itcf_slashleft_twohanded

#EXtra Mesh IDs
ixmesh_inventory   = 0x1000000000000000
ixmesh_flying_ammo = 0x2000000000000000
ixmesh_carry       = 0x3000000000000000




## 用于编写一些ai方面的功能
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_skills import *
from smart_module.module_system.module_constants import *

missionBaseScripts={
    "name":"AiBaseScripts",
    "enable":True,
    "version": "v2.0.0",
    "desc":"与场景或战场相关的操作",
    "actions":[
        ("Append@scripts",[
            ("create_battle_for_player", [
                (store_script_param_1, ":center"),
                (store_script_param_2, ":place"),
                (store_script_param, ":companies_nums", 3),
                (store_script_param, ":enemies_nums", 4),

                # (party_get_slot, ":scene_no", "p_town_1", slot_town_alley),
                (try_begin),
                (gt, ":place", 0),
                (party_get_slot, ":scene_no", ":center", ":place"),
                (else_try),
                (assign, ":scene_no", ":center"),
                (try_end),

                (modify_visitors_at_site, ":scene_no"),

                (reset_visitors),
                (set_visitor, 0, "trp_player"),

                (party_add_members, "p_main_party", "trp_swadian_knight", ":companies_nums"),

                (troop_add_item, "trp_player", "itm_saddle_horse", 0),
                (troop_add_item, "trp_player", "itm_courser", 0),
                (troop_add_item, "trp_player", "itm_courtly_outfit", 0),
                (troop_add_item, "trp_player", "itm_heraldic_mail_with_tabard", 0),
                (troop_add_item, "trp_player", "itm_red_gambeson", 0),
                (troop_add_item, "trp_player", "itm_sword_medieval_c", 0),
                (troop_add_item, "trp_player", "itm_tab_shield_kite_cav_b", 0),
                (troop_add_item, "trp_player", "itm_light_lance", 0),
                (troop_raise_skill, "trp_player", skl_riding, 10),
                (troop_equip_items, "trp_player"),

                (val_mul, ":enemies_nums", 3),
                (set_visitors, 2, "trp_bandit", ":enemies_nums"),

                (set_jump_mission, "mt_alley_fight"),
                (jump_to_scene, ":scene_no"),
                (change_screen_mission),
            ]),
            ("go_to_center", [
                ## 地点  -1代表大地图
                (store_script_param_1, ":center"),
                ## 地区 酒馆，街道等等  -1代表 村庄中心（村庄只有一个地区）
                (store_script_param_2, ":place"),
                (store_script_param, ":gold_amonut", 3),
                (store_script_param, ":renown_amonut", 4),
                (store_script_param, ":has_companion", 5),
                (store_script_param, ":has_soldier", 6),
                (store_script_param, ":has_prisoner", 7),

                ## 名称
                (troop_set_name, "trp_player", "@test"),
                (party_set_name, "p_main_party", "@test"),
                ## 基础物品
                (troop_add_item, "trp_player", "itm_saddle_horse", 0),
                (troop_add_item, "trp_player", "itm_courser", 0),
                (troop_add_item, "trp_player", "itm_courtly_outfit", 0),
                (troop_add_item, "trp_player", "itm_heraldic_mail_with_tabard", 0),
                (troop_add_item, "trp_player", "itm_red_gambeson", 0),
                (troop_add_item, "trp_player", "itm_sword_medieval_c", 0),
                (troop_add_item, "trp_player", "itm_tab_shield_kite_cav_b", 0),
                (troop_add_item, "trp_player", "itm_light_lance", 0),
                (troop_add_items, "trp_player", "itm_dried_meat", 10),
                ## 穿上装备
                (troop_equip_items, "trp_player"),

                ## 基础技能
                (troop_raise_skill, "trp_player", skl_riding, 10),
                (troop_raise_skill, "trp_player", skl_leadership, 10),
                (troop_raise_attribute, "trp_player", sf_base_att_str, 10),
                (troop_raise_attribute, "trp_player", sf_base_att_agi, 100),
                (troop_raise_attribute, "trp_player", sf_base_att_int, 10),
                (troop_raise_attribute, "trp_player", sf_base_att_cha, 10),

                ## 金币
                (troop_add_gold, "trp_player", ":gold_amonut"),
                ## 声望
                (troop_set_slot, "trp_player", slot_troop_renown, ":renown_amonut"),
                ## 士兵
                (try_begin),
                (gt, ":has_soldier", 0),
                (party_add_members, "p_main_party", "trp_swadian_knight", 70),
                (try_end),
                ## 同伴
                (try_begin),
                (gt, ":has_companion", 0),
                (try_for_range, ":npc", companions_begin, companions_end),
                (call_script, "script_recruit_troop_as_companion", ":npc"),
                (troop_raise_skill, ":npc", skl_persuasion, 1),
                (try_end),
                (try_end),
                ## 奴隶
                (try_begin),
                (gt, ":has_prisoner", 0),
                (troop_raise_skill, "trp_player", skl_prisoner_management, 10),
                (party_add_prisoners, "p_main_party", "trp_swadian_knight", 10),
                (try_end),

                (try_begin),
                (eq, ":center", -1),
                (party_relocate_near_party, "p_main_party", "p_village_1", 3),
                (change_screen_map),
                (else_try),
                (assign, "$current_town", ":center"),
                (try_begin),
                (eq, ":scene_no", -1),
                (party_get_slot, ":scene_no", ":center", slot_castle_exterior),
                (set_jump_mission, "mt_village_center"),
                (jump_to_scene, ":scene_no"),
                (else_try),
                (party_get_slot, ":scene_no", ":center", ":place"),
                (set_jump_mission, "mt_town_center"),
                (jump_to_scene, ":scene_no"),
                (try_end),
                ## 更新访问者
                (modify_visitors_at_site, ":scene_no"),
                (reset_visitors),
                ## 设置村长
                (party_get_slot, ":village_elder_troop", ":center", slot_town_elder),
                (set_visitor, 11, ":village_elder_troop"),
                ## 初始化村民
                (call_script, "script_init_town_walkers"),
                (change_screen_mission),
                (try_end),
            ]),
        ])
    ],
}


###################################################
# header_paricle_systems.py
# This file contains declarations for particle_systems
# DO NOT EDIT THIS FILE!
###################################################

psf_always_emit         = 0x0000000002
psf_global_emit_dir     = 0x0000000010
psf_emit_at_water_level = 0x0000000020
psf_billboard_2d        = 0x0000000100 # up_vec = dir, front rotated towards camera
psf_billboard_3d        = 0x0000000200 # front_vec point to camera.
psf_billboard_drop      = 0x0000000300
psf_turn_to_velocity    = 0x0000000400
psf_randomize_rotation  = 0x0000001000
psf_randomize_size      = 0x0000002000
psf_2d_turbulance       = 0x0000010000
psf_next_effect_is_lod  = 0x0000020000



###################################################
# header_skins.py
# This file contains declarations for skins
# DO NOT EDIT THIS FILE!
###################################################

voice_die        = 0
voice_hit        = 1
voice_grunt      = 2
voice_grunt_long = 3
voice_yell       = 4
voice_warcry     = 5
voice_victory    = 6
voice_stun       = 7

skf_use_morph_key_10 = 0x00000001
skf_use_morph_key_20 = 0x00000002
skf_use_morph_key_30 = 0x00000003
skf_use_morph_key_40 = 0x00000004
skf_use_morph_key_50 = 0x00000005
skf_use_morph_key_60 = 0x00000006
skf_use_morph_key_70 = 0x00000007




## 包含一些对兵种常用的操作

## 姓氏
from smart_module.module_system.header_operations import *
from smart_module.module_system.header_parties import *
from smart_module.module_system.header_skills import *
from smart_module.module_system.module_constants import *
from smart_module.smart_core.utils.pythonUtils import *
from smart_module.smart_core.smart_module_slot import smartModuleSlotManager

first = ["陈","李","黄","张","朱","梁","林","刘","马","白","吴","曹","蔡","车","谭","罗","杨","诸 葛","司 马","南 宫","太 史","公 良","公 孙","公 良","百 里"]
## 名字
second = ["宛","丘","形","采","用","其","利","器","用","桐","平","桃","山","明","月","朋","峰","峦","清","浮","桥","杰","贡","英","弄","三","武","言","玄 策","大 目","三 刀"]


def parseFirstStrings(chars):
    list = []
    for index in range(len(chars)):
        list.append(("first_name_{}".format(index),"{}".format(chars[index])))
    list.append(("first_name_end","end"))
    return list

def parseSecondStrings(chars):
    list = []
    for index in range(len(chars)):
        list.append(("second_name_{}".format(index),"{}".format(chars[index])))
    list.append(("second_name_end","end"))
    return list

def parseCnsFirstStrings(chars):
    list = []
    for index in range(len(chars)):
        list.append("str_first_name_{}|{}".format(index,chars[index]))
    return list

def parseCnsSecondStrings(chars):
    list = []
    for index in range(len(chars)):
        list.append("str_second_name_{}|{}".format(index,chars[index]))
    return list

first_begin = "str_first_name_0"
first_end = "str_first_name_end"

second_begin = "str_second_name_0"
second_end = "str_second_name_end"



## slot
slot_troop_first_name = smartModuleSlotManager.getTroopSlotNo("slot_troop_first_name")
slot_troop_second_name = smartModuleSlotManager.getTroopSlotNo("slot_troop_second_name")

troopBaseScripts={
    "name":"TroopBaseScripts",
    "enable":True,
    "version": "v2.0.0",
    "desc":"与人物或兵种相关的操作",
    "actions": [
        ("Append@scripts",[
            ## 出门
            ("troop_leave_home",[
                (store_script_param, ":troop", 1),
                (troop_get_slot,":home",":troop",slot_troop_cur_center),
                (party_get_slot, ":home_castle", ":home", slot_town_castle),
                (remove_troop_from_site, ":troop",":home_castle"),
                ##(display_message,"str_s5_leave_home"),
              ]),
            ## 回家
            ("troop_go_home",[
                (store_script_param, ":troop", 1),
                (troop_get_slot,":home",":troop",slot_troop_cur_center),
                (party_get_slot, ":home_castle", ":home", slot_town_castle),
                (modify_visitors_at_site, ":troop",":home_castle"),
                ##(display_message,"str_s5_go_home"),
              ]),

            ("set_random_name",[
                (store_script_param_1,":troop"),
                (store_random_in_range,":first",first_begin,first_end),
                (store_random_in_range,":second",second_begin,second_end),
                (troop_set_slot,":troop",slot_troop_first_name,":first"),
                (troop_set_slot,":troop",slot_troop_second_name,":second"),
                (str_store_string,s1,":first"),
                (str_store_string,s2,":second"),
                (troop_set_name,":troop","str_s1_s2_name"),
            ]),
            ("get_random_first_name",[
                (store_random_in_range,":first",first_begin,first_end),
                (assign,reg0,":first"),
            ]),
            ("get_random_second_name", [
                (store_random_in_range, ":second", second_begin, second_end),
                (assign,reg0,":second"),
            ]),

            ("set_name_for_son", [
                (store_script_param_1, ":father"),
                (store_script_param_2, ":son"),

                (str_store_troop_name,s3,":father"),

                (troop_get_slot,":first_name",":father",slot_troop_first_name),

                (assign,reg1,":first_name"),
                (display_message,"@father first name id {reg1}"),


                (call_script,"script_get_random_second_name"),
                (assign,":second_name",reg0),

                (troop_set_slot,":son",slot_troop_first_name,":first_name"),
                (troop_set_slot,":son",slot_troop_second_name,":second_name"),

                (str_store_string, s1, ":first_name"),
                (str_store_string, s2, ":second_name"),
                (troop_set_name, ":son", "str_s1_s2_name"),

                (troop_set_slot, ":son", slot_troop_father, ":father"),


                (display_message,"@son(father:{s3}) first name:{s1}  second name:{s2}"),
            ]),
            ("display_troop_info",[
                (store_script_param_1,":troop"),
                ## 姓名
                (str_store_troop_name,s1,":troop"),
                (display_message,"@name:{s1}"),
                ## 姓
                (troop_get_slot,":first_name",":troop",slot_troop_first_name),
                (str_store_string,s1,":first_name"),
                (display_message,"@firstName:{s1}"),
                ## 名
                (troop_get_slot,":second_name",":troop",slot_troop_second_name),
                (str_store_string,s1,":second_name"),
                (display_message,"@secondName:{s1}"),
                ## 年龄
                (troop_get_slot,reg1,":troop",slot_troop_age),
                (display_message,"@age:{reg1}"),
                ## 性别
                (troop_get_type,reg1,":troop"),
                (display_message,"@sex:{reg1?str_man:str_woman}"),
                ## 财富
                (troop_get_slot,reg1,":troop",slot_troop_wealth),
                (display_message,"@wealth:{reg1}"),
                ## 被俘虏城市
                (troop_get_slot,":center",":troop",slot_troop_prisoner_of_party),
                (try_begin),
                    (gt,":center",0),
                    (str_store_party_name,s1,":center"),
                    (display_message,"@prisoner:{s1}"),
                (try_end),

                (display_message,"@-----------------------------------------------------"),

            ]),
            ("set_age_in_range",[
                (store_script_param,":troop",1),
                (store_script_param,":min_age",2),
                (store_script_param,":max_age",3),
                (store_random_in_range, ":age", ":min_age", ":max_age"),
                (call_script, "script_init_troop_age", ":troop", ":age"),
            ]),
            ("set_son_age",[
                (store_script_param,":father",1),
                (store_script_param,":son",2),
                (troop_get_slot,":father_age",":father",slot_troop_age),
                (store_random_in_range,":father_age_in_son_birth",20,30),
                (store_sub,":age",":father_age",":father_age_in_son_birth"),
                (call_script, "script_init_troop_age", ":son", ":age"),
            ]),
            ("troop_clear_items",[
                (store_script_param,":troop",1),
                (troop_get_inventory_capacity, ":inv_size", ":troop"),
                (try_for_range, ":i_slot", 0, ":inv_size"),
                    (troop_get_inventory_slot, ":item_id", ":troop", ":i_slot"),
                    (ge, ":item_id", 0),
                    (troop_remove_item,":troop",":item_id"),
                (try_end),
            ]),
            ("get_troop_all_items",[
                (store_script_param,":target_troop",1),
                (store_script_param,":source_troop",2),
                (call_script,"script_troop_clear_items",":target_troop"),

                (troop_get_inventory_capacity, ":inv_size", ":source_troop"),
                (try_for_range, ":i_slot", 0, ":inv_size"),
                    (troop_get_inventory_slot, ":item_id", ":source_troop", ":i_slot"),
                    (ge, ":item_id", 0),
                    (troop_add_item,":target_troop",":item_id"),
                (try_end),
            ]),
            ("get_troop_all_wealth",[
                (store_script_param,":target_troop",1),
                (store_script_param,":source_troop",2),
                (troop_get_slot,":wealth",":source_troop",slot_troop_wealth),
                (troop_get_slot,":renown",":source_troop",slot_troop_renown),
                (troop_get_slot,":father",":source_troop",slot_troop_father),
                (troop_get_slot,":mother",":source_troop",slot_troop_mother),
                (troop_get_slot,":spouse",":source_troop",slot_troop_spouse),
                (troop_get_slot,":guardian",":source_troop",slot_troop_guardian),
                (troop_get_slot,":betrothed",":source_troop",slot_troop_betrothed),

                (troop_set_slot,":target_troop",slot_troop_wealth,":wealth"),
                (troop_set_slot,":target_troop",slot_troop_renown,":renown"),
                (troop_set_slot,":target_troop",slot_troop_father,":father"),
                (troop_set_slot,":target_troop",slot_troop_mother,":mother"),
                (troop_set_slot,":target_troop",slot_troop_spouse,":spouse"),
                (troop_set_slot,":target_troop",slot_troop_guardian,":guardian"),
                (troop_set_slot,":target_troop",slot_troop_betrothed,":betrothed"),

                (store_faction_of_troop,":faction",":source_troop"),
                (try_begin),
                    (faction_slot_eq,":faction",slot_faction_leader,":source_troop"),
                    (faction_set_slot,":faction",slot_faction_leader,":target_troop"),
                (try_end),

                (call_script, "script_update_troop_notes",":target_troop"),
            ]),
            ("player_cosplay_anyone",[
                (store_script_param,":troop",1),

                (str_store_troop_name,s5,":troop"),
                (display_message,"@cosplay {s5}"),

                (troop_set_name,"trp_player",s5),
                (party_set_name,"p_main_party","str_s5_s_party"),

                (store_faction_of_troop,":faction",":troop"),
                (call_script,"script_player_join_faction",":faction"),

                (assign, "$player_has_homage", 1),

                (call_script,"script_get_center_of_lord",":troop","p_town_1"),
                (assign,":center",reg0),

                (call_script,"script_get_troop_all_items","trp_player",":troop"),

                (troop_equip_items,"trp_player"),
                (call_script,"script_get_troop_all_wealth","trp_player",":troop"),
                (call_script,"script_get_loard_all_centers","trp_player",":troop"),
                (troop_get_slot,":party",":troop",slot_troop_leaded_party),
                (call_script,"script_add_party_as_companions","p_main_party",":party",-1),
                (party_relocate_near_party,"p_main_party",":center",3),
                ## 设置被替换的人物为不启用
                (troop_set_slot,":troop",slot_troop_occupation,slto_inactive),
                (call_script,"script_update_all_notes"),
                ##(assign, "$current_startup_quest_phase", 4),
            ]),
            ## 获得领主的军事强度
            ("get_lord_strength",[
                (store_script_param_1,":leader"),
                (assign,":strength",0),
                (try_begin),
                    (ge,":leader"),
                    ## 战术加成
                    (store_skill_level,":tactics",skl_tactics,":leader"),
                    (store_mul,":tactics_strength",":tactics",2),
                    (val_add,":strength",":tactics_strength"),
                    ## 教练加成
                    (store_skill_level,":trainer",skl_trainer,":leader"),
                    (store_mul,":trainer_strength",":trainer",2),
                    (val_add,":strength",":trainer_strength"),
                    ## 性格加成
                    (this_or_next|troop_slot_eq,":leader",slot_lord_reputation_type,lrep_martial),
                    (troop_slot_eq,":leader",slot_lord_reputation_type,lrep_quarrelsome),
                    (val_add,":strength",3),
                    ## 性格缺陷
                    (troop_slot_eq,":leader",slot_lord_reputation_type,lrep_custodian),
                    (val_sub,":strength",3),
                (try_end),
                (assign,reg0,":strength"),
            ]),
            ## 更新领主财富
            ("update_lord_wealth",[
                 (store_script_param_1, ":lord_no"),
                 (store_script_param_2, ":value"),
                 ## 0:失去钱 1：获得钱
                 (store_script_param, ":type",3),

                 (troop_get_slot,":wealth",":lord_no",slot_troop_wealth),
                 (try_begin),
                     (gt,":type",0),
                     (val_add,":wealth",":value"),
                 (else_try),
                    (val_sub,":wealth",":value"),
                 (try_end),
                 (troop_set_slot,":lord_no",slot_troop_wealth,":wealth"),
             ]),
            ("get_troop_inventory_free_amount",
                [(store_script_param, ":troop_no", 1),
                 (troop_get_inventory_capacity, ":inv_cap", ":troop_no"),
                 (assign, ":count", 0),
                 ## 前边10个是人物的装备栏（有可能第0个不是，只有9个）
                 (try_for_range, ":i_slot", 10, ":inv_cap"),
                   (troop_get_inventory_slot, ":cur_item", ":troop_no", ":i_slot"),
                   (eq, ":cur_item", -1),
                   (val_add, ":count", 1),
                 (try_end),
                 (assign, reg0, ":count"),
            ]),
        ]),
        ("Append@strings",mergeList(
            parseFirstStrings(first),
            parseSecondStrings(second),
            [
                ("s5_leave_home","{s5} leave home"),
                ("s5_go_home","{s5} go home"),
                ("s1_s2_name","{s1} {s2}"),
            ],
        ))
    ],
    "internationals":{
        "cns":{
            "game_strings":mergeList(
                parseCnsFirstStrings(first),
                parseCnsSecondStrings(second),
                [
                    "str_s5_leave_home|{s5}出 门 了",
                    "str_s5_go_home|{s5}回 家 了",
                    "str_s1_s2_name|{s1}{s2}",
                ]
            ),
        }
    }
}